{% extends 'generic/object_children.html' %}

{% block extra_controls %}
{% if perms.ipfabric_netbox.add_ipfabricrelationshipfield %}
<a href="{% url 'plugins:ipfabric_netbox:ipfabricrelationshipfield_add' %}?transform_map={{ object.pk }}&return_url={% url 'plugins:ipfabric_netbox:ipfabrictransformmap_relationships' pk=object.pk %}"
    class="btn btn-primary btn-sm float-end">
    <i class="mdi mdi-plus-thick" aria-hidden="true"></i> Relationship Map
</a>
{% endif %}
{{ block.super }}
{% endblock extra_controls %}
