{% extends 'generic/object.html' %}
{% load static %}
{% load helpers %}
{% load plugins %}
{% load render_table from django_tables2 %}

{% block content %}
<div class="row mb-3">
  <div class="col col-md-6">
    <div class="card">
      <h5 class="card-header">Transform Map Information</h5>
      <div class="card-body">
        <table class="table table-hover attr-table">
          <tr>
            <th scope="row">Name</th>
            <td>{{ object.name }}</td>
          </tr>
          <tr>
            <th scope="row">Source Model</th>
            <td>ipfabric | {{ object.source_model }}</td>
          </tr>
          <tr>
            <th scope="row">Target Model</th>
            <td>{{ object.target_model }}</td>
          </tr>
          <tr>
            <th scope="row">Status</th>
            <td>{{ object.status }}</td>
          </tr>
          </tr>
        </table>
      </div>
    </div>
    {% plugin_left_page object %}
  </div>
  <div class="col col-md-6">
    {% include 'inc/panels/custom_fields.html' %}
    {% plugin_right_page object %}
  </div>
</div>
{% endblock %}
