# Generated by Django 4.1.10 on 2024-01-05 16:59
import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("ipfabric_netbox", "0002_ipfabricsnapshot_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="ipfabricsource",
            name="type",
            field=models.CharField(default="local", max_length=50),
        ),
        migrations.AlterField(
            model_name="ipfabricsnapshot",
            name="snapshot_id",
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name="ipfabricsource",
            name="last_synced",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name="IPFabricData",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("data", models.JSONField(blank=True, null=True)),
                ("type", models.CharField(max_length=50)),
                (
                    "snapshot_data",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ipf_data",
                        to="ipfabric_netbox.ipfabricsnapshot",
                    ),
                ),
            ],
        ),
    ]
