{# Indicates that a job result is still pending; used for HTMX requests #}
<div class="spinner-border float-start me-1" id="spinner" role="status">
  <span class="visually-hidden">Loading...</span>
</div>
<h3>Logs pending...</h3>
<small class="text-muted">Last updated {% now "H:i:s" %}</small>
