
<div id="sync_status" hx-swap-oob="true">
  {% include 'ipfabric_netbox/partials/branch_status.html' with object=object %}
</div>
<div id="sync_progress" hx-swap-oob="true">
  {% include 'ipfabric_netbox/partials/branch_progress.html' %}
</div>
<div id="sync_logs">
  {% include 'ipfabric_netbox/partials/job_logs.html' with job_results=job_results job=job %}
</div>
